//
//  PCPreferencesController.h
//  PanicCore
//
//  Created by Logan Collins on 2/27/12.
//  Copyright (c) 2012 Panic Inc. All rights reserved.
//

#import <Cocoa/Cocoa.h>


@class PCPreferencePane, PCPreferencesWindow, PCPreferencesContainerView;


/*!
 * @class PCPreferencesController
 * @abstract A preferences window controller
 *
 * @discussion
 * The controller manages a series of panes that are displayed as a preferences window.
 */
@interface PCPreferencesController : NSResponder {
@private
    PCPreferencesWindow *_window;
    PCPreferencesContainerView *_containerView;
    
    NSMutableArray *_preferencePaneIdentifiers;
    NSMutableDictionary *_preferencePanes;
    NSMutableDictionary *_toolbarItems;
    
    PCPreferencePane *_currentPane;
    PCPreferencePane *_disappearingPane;
    BOOL _animating;
    void (^_animationCompletionHandler)(void);
    
    NSToolbar *_toolbar;
}

/*!
 * @method sharedController
 * @abstract Gets the default preferences controller
 *
 * @result A PCPreferencesController object
 */
+ (PCPreferencesController *)defaultController;


/*!
 * @property autosaveIdentifier
 * @abstract The preferences controller autosave identifier
 *
 * @result An NSString object
 */
@property (copy) NSString *autosaveIdentifier;

/*!
 * @property window
 * @abstract The preferences controller window
 *
 * @result An NSWindow object
 */
@property (strong, readonly) NSWindow *window;


/*!
 * @property preferencePanes
 * @abstract Gets the array of preference panes.
 *
 * @result An NSArray of PCPreferencePane objects
 */
@property (strong, readonly) NSArray *preferencePanes;

/*!
 * @property currentPreferencePane
 * @abstract Gets the current preference pane.
 *
 * @result A PCPreferencePane object
 */
@property (strong, readonly) PCPreferencePane *currentPreferencePane;

/*!
 * @method preferencePaneWithIdentifier:
 * @abstract Gets the preference pane with a specific identifier (or nil if none exists)
 *
 * @param identifier
 * The identifier of the preference pane to get
 *
 * @result A PCPreferencePane object
 */
- (PCPreferencePane *)preferencePaneWithIdentifier:(NSString *)identifier;

/*!
 * @method preferencePaneAtIndex:
 * @abstract Gets the preference pane at a specific index
 *
 * @param index
 * The index of the preference pane to get
 *
 * @result A PCPreferencePane object
 */
- (PCPreferencePane *)preferencePaneAtIndex:(NSUInteger)index;

/*!
 * @method addPreferencePane:
 * @abstract Append a preference pane to the receiver
 *
 * @param preferencePane
 * The preference pane to add
 */
- (void)addPreferencePane:(PCPreferencePane *)preferencePane;

/*!
 * @method insertPreferencePane:atIndex:
 * @abstract Append a preference pane to the receiver
 *
 * @param preferencePane
 * The preference pane to insert
 *
 * @param index
 * The index at which to insert the preference pane
 */
- (void)insertPreferencePane:(PCPreferencePane *)preferencePane atIndex:(NSUInteger)index;

/*!
 * @method removePreferencePaneAtIndex:
 * @abstract Removes the preference pane at a specified index from the receiver
 *
 * @param index
 * The index of the preference pane to remove
 */
- (void)removePreferencePaneAtIndex:(NSUInteger)index;

/*!
 * @method removePreferencePane:
 * @abstract Removes a specified preference pane from the receiver
 *
 * @param preferencePane
 * The preference pane to remove
 */
- (void)removePreferencePane:(PCPreferencePane *)preferencePane;


/*!
 * @method showPreferencePaneWithIdentifier:
 * @abstract Switch to the preference pane with a specified identifier
 *
 * @param identifier
 * The identifier of the pane to which to switch
 */
- (void)showPreferencePaneWithIdentifier:(NSString *)identifier;


/*!
 * @method showWindow:
 * @abstract Shows the receiver's window
 *
 * @param sender
 * The action sender
 */
- (IBAction)showWindow:(id)sender;

@end
